// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/driver/main.h $
// $Rev: 52446 $
// $Date: 2023-01-30 13:41:09 -0600 (Mon, 30 Jan 2023) $

// ADADIO: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "adadio.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"ADADIO"	// Upper case form of the below.
#define	DEV_NAME							"adadio"	// MUST AGREE WITH ADADIO_BASE_NAME

#define	DEV_VERSION							"4.9"		// FOR DEVICE SPECIFIC CODE ONLY!
// 4.9	Updated comments for consistency.
//		Updated the Initialize and Autocal IOCTL services for consistency.
//		Deleted unused macros.
//		Updated per PLX code segregation.
// 4.8	Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
//		LINTed sources.
//		Fiaxed bugs in the DIO Pin read and write services.
// 4.7	Corrected ADADIO2 detection method.
// 4.6	Added support for the ADADIO2.
//		Corrected the definition of the BRR register macro.
// 4.5	Added support for the 5.x kernel series.
//		Added Endianness support.
//		Removed duplicate spinlock initialization code.
//		Modified the last query option value macro.
// 4.4	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Modified code so open succeeds even if initialization fails.
// 4.3	Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
//		Corrected the IOCTL code for the register write and mod services.
// 4.2	Added the ADADIO_IOCTL_AIN_BUF_ENABLE service.
// 4.1	details not documented (oops)
// 4.0	Updated to use the newer common driver sources.
//		Reading and caching of the BRR to now done when the driver is loaded.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		Updated gsc_dma_open() and gsc_dma_close().
// 3.9	Modified to use more of the newer common sources.
//		Minor update the DMA Read data availability service.
// 3.8	The initialization routine no longer uses interrupts.
//		Device open requests ignore initialization failures.
// 3.7	Reduced #include list in driver interface header.
// 3.6	Updated for the 3.x kernel.
//		Corrected a system timer count roll over bug in ioctl.c.
// 3.5	BAR0 and BAR2 are now the only BAR regions used.
//		Include all common source, though not all are used.
// 3.4	Eliminated the global dev_check_id() routine.
//		Changed data type for dev_data_t.sem.
//		Updated the ADADIO_IOCTL_AIN_BUF_CLEAR service.
// 3.3	Overhauled IOCTL based local interrupt implementation.
//		Updated per changes to the common source code.
//		Implemented IOCTL based I/O Abort support.
//		Implemented IOCTL based event/interrupt wait support.
//		Updated to use the common gsc_open() and gsc_close() services.
//		Implemented explicit local interrupt support.
//		Implemented support for the common PIO I/O routines.
//		Changed use of DEV_SUPPORTS_READ macro.
//		Changed use of DEV_SUPPORTS_WRITE macro.
//		Added support for the older 9060 based ADADIO devices.
//		Made the ADADIO_IOCTL_INITIALIZE service more efficient and robust.
//		Made the ADADIO_IOCTL_AUTO_CALIBRATE service more efficient and robust.
// 3.2	Modified open/close services to use GSC common code.
// 3.1	Fixed detection logic for device's DMDMA support.
//		Modified I/O services to use common argument types.
//		Added dev_check_id() service.
// 3.0	Overhauled driver.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				0

#define	GSC_READ_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					ADADIO_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					ADADIO_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						ADADIO_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					io_mode;			// PIO, DMA, DMDMA
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.

	// Initialized by I/O service at time of use.

	int					abort;				// Also set via IOCTL service.
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	// Put an integral data type at beginning of structure.
	int					board_index;	// Index of the device being accessed.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					share;			// Were we opened in shared mode?
	int					users;			// Number of currently active open() requests.

	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrupts.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;				// For read operations.
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_bcr_32;		// 0x00 Board Control Register
		VADDR_T			gsc_diopr_32;	// 0x04 Digital I/O Port Register
		VADDR_T			gsc_aoc0r_32;	// 0x08 Analog Output Ch 0 Register
		VADDR_T			gsc_aoc1r_32;	// 0x0C Analog Output Ch 1 Register
		VADDR_T			gsc_aoc2r_32;	// 0x10 Analog Output Ch 2 Register
		VADDR_T			gsc_aoc3r_32;	// 0x14 Analog Output Ch 3 Register
		VADDR_T			gsc_aidr_32;	// 0x18 Analog Input Data Register
		VADDR_T			gsc_srr_32;		// 0x1C Sample Rate Register
	} vaddr;

	struct
	{
		s32				auto_cal_ms;
		u32				brr;			// Board Revision Register
		s32				dmdma;			// Is Demand Mode DMA supported?
		s32				fifo_size;
		s32				fsamp_max;
		s32				fsamp_min;
		s32				initialize_ms;
		s32				master_clock;
		s32				rate_gen_nrate_mask;
		s32				rate_gen_nrate_max;
		s32				rate_gen_nrate_min;
		s32				vrange;
	} cache;

};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t bytes);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);

int			dev_read_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
