// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/samples/sbtest/wait_event.c $
// $Rev: 52466 $
// $Date: 2023-01-30 13:55:32 -0600 (Mon, 30 Jan 2023) $

// ADADIO: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1_SEC		(1000L)
#define	_20_SEC		(20L * 1000L)
#define	_30_SEC		(30L * 1000L)



// variables ******************************************************************

static	int			io_stat;
static	sem_t		sem_enter;
static	sem_t		sem_exit;
static	gsc_wait_t	wait_dst;
static	int			wait_fd;



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no settings bits to test here.
	return(0);
}



//*****************************************************************************
static int	_wait_main_dma(int fd)
{
	static const service_data_t	list[]	=
	{
		{					// Select DMA mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_BMDMA,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_MODE,
			/* arg		*/	ADADIO_AIN_MODE_DIFF_CONT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_BUF_SIZE,
			/* arg		*/	ADADIO_AIN_BUF_SIZE_128,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_NRATE,
			/* arg		*/	200,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait to see if we get any data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[100];
	int	errs	= 0;
	s32	ret;

	errs	+= adadio_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, list);

	// Now perform a DMA read to trigger the wait.

	if (errs == 0)
	{
		ret		= adadio_read(fd, buf, sizeof(buf));
		errs	+= (ret == sizeof(buf)) ? 0 : 1;
	}

	errs	+= adadio_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int	_wait_main_gsc(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_init_done(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_auto_cal_done(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_ain_buf_empty(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_IRQ_SEL,
			/* arg		*/	ADADIO_IRQ_AIN_BUF_EMPTY,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_NRATE,
			/* arg		*/	0xFFFF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_MODE,
			/* arg		*/	ADADIO_AIN_MODE_DIFF_CONT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_ain_buf_half_full(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_IRQ_SEL,
			/* arg		*/	ADADIO_IRQ_AIN_BUF_HALF_FULL,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_MODE,
			/* arg		*/	ADADIO_AIN_MODE_DIFF_CONT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_BUF_SIZE,
			/* arg		*/	ADADIO_AIN_BUF_SIZE_32,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_ain_buf_full(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_IRQ_SEL,
			/* arg		*/	ADADIO_IRQ_AIN_BUF_FULL,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_MODE,
			/* arg		*/	ADADIO_AIN_MODE_DIFF_CONT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_BUF_SIZE,
			/* arg		*/	ADADIO_AIN_BUF_SIZE_32,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_ain_burst_done(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_IRQ_SEL,
			/* arg		*/	ADADIO_IRQ_AIN_BURST_DONE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_MODE,
			/* arg		*/	ADADIO_AIN_MODE_DIFF_BURST,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int	_wait_gsc_aout_strobe_done(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_IRQ_SEL,
			/* arg		*/	ADADIO_IRQ_AOUT_STROBE_DONE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AOUT_ENABLE,
			/* arg		*/	ADADIO_AOUT_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AOUT_STROBE_ENABLE,
			/* arg		*/	ADADIO_AOUT_STROBE_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AOUT_STROBE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, list);
	return(errs);
}



//*****************************************************************************
static int	_wait_io_rx_done(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_AIN_MODE,
			/* arg		*/	ADADIO_AIN_MODE_DIFF_CONT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[10];
	int	errs;
	int	ret;

	errs	= service_ioctl_set_reg_list(fd, list);

	// Perform a read to trigger the wait.

	if (errs == 0)
	{
		ret	= adadio_read(fd, buf, sizeof(buf));

		if (ret < 0)
		{
			errs++;
			printf("FAIL <---  (adadio_read() returned %d)\n", ret);
		}
		else if (ret != sizeof(buf))
		{
			errs++;
			printf(	"FAIL <---  (adadio_read(): returned %d, expected %d)\n",
					ret,
					(int) sizeof(buf));
		}
	}

	return(errs);
}



//*****************************************************************************
static int	_wait_io_rx_error(int fd)
{
	int	errs	= 0;
	int	ret;

	ret	= adadio_read(fd, NULL, 3);

	if (ret >= 0)
	{
		errs++;
		printf("FAIL <---  (adadio_read(): expected < 0, got %d)\n", (int) ret);
	}

	return(errs);
}



//*****************************************************************************
static int	_wait_io_rx_timeout(int fd)
{
	static const service_data_t	list[]	=
	{
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1024];
	int	errs;
	int	sts;

	errs	= service_ioctl_set_reg_list(fd, list);

	// Perform a read to trigger the wait.

	if (errs == 0)
	{
		sts	= adadio_read(fd, buf, sizeof(buf));

		if (sts != 0)
		{
			errs++;
			printf(	"FAIL <---  (expected %d, got %d)\n",
					(int) 0,
					(int) sts);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _read_thread(void* arg)
{
	u32	buf[1024];
	int	fd	= (int) (long) arg;
	int	sts;

	sts	= adadio_read(fd, buf, sizeof(buf));

	if (sts == 0)
	{
		io_stat	= 1;
	}
	else
	{
		io_stat	= 2;
		printf(	"FAIL <---  (adadio_read(): expected %d, got %d)\n",
				(int) 0,
				(int) sts);
	}

	return(0);
}



//*****************************************************************************
static int	_wait_io_rx_abort(int fd)
{
	static const service_data_t	list[]	=
	{
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	ADADIO_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	int			i;
	int			ret;
	s32			set;
	os_thread_t	thread;

	errs	+= service_ioctl_set_reg_list(fd, list);

	// Start the read thread so we can do the abort.
	memset(&thread, 0, sizeof(thread));

	if (errs == 0)
	{
		errs	+= os_thread_create(	&thread,
										"Rx",
										_read_thread,
										(void*) (long) fd);
	}

	for (i = 0; (errs == 0) && (i < 100); i++)
	{
		set		= 0;
		ret		= adadio_ioctl(fd, ADADIO_IOCTL_RX_IO_ABORT, &set);
		errs	+= ret ? 1 : 0;

		if (errs)
			break;

		if (set)
			break;

		os_sleep_ms(100);
	}

	// Wait for the thread to report its status.

	for (i = 0; i < 100; i++)
	{
		if (errs)
			break;

		if (io_stat == 0)
		{
			os_sleep_ms(100);
			continue;
		}

		if (io_stat == 1)
			break;

		errs++;
		break;
	}

	errs	+= os_thread_destroy(&thread);
	return(errs);
}



//*****************************************************************************
static int	_wait_timeout(int fd)
{
	os_sleep_ms(2000);
	return(0);
}



//*****************************************************************************
static int	_wait_cancel(int fd)
{
	int			errs;
	int			ret;
	gsc_wait_t	wait;

	wait.flags			= 0;
	wait.main			= 0xFFFFFFFF;
	wait.gsc			= 0xFFFFFFFF;
	wait.alt			= 0xFFFFFFFF;
	wait.io				= 0xFFFFFFFF;
	wait.timeout_ms		= 0;
	wait.count			= 0;
	ret		= adadio_ioctl(wait_fd, ADADIO_IOCTL_WAIT_CANCEL, &wait);
	errs	= ret ? 1 : 0;
	return(errs);
}



// variables ******************************************************************

static const struct
{
	const char*	name;
	int			(*function)(int fd);
	gsc_wait_t	wait;
} wait_src[]	=
{
	// name						function						flags			main								...			timeout_ms
	{ "DMA0/DMA1",				_wait_main_dma,					{ 0,			GSC_WAIT_MAIN_DMA0 |
																				GSC_WAIT_MAIN_DMA1,					0, 0, 0,	_20_SEC	}	},
	{ "GSC",					_wait_main_gsc,					{ 0,			GSC_WAIT_MAIN_GSC,					0, 0, 0,	_20_SEC	}	},

	// name						function						flags ...		gsc									...			timeout_ms
	{ "Initialize Done",		_wait_gsc_init_done,			{ 0, 0,			ADADIO_WAIT_GSC_INIT_DONE,			0, 0,		_20_SEC	}	},
	{ "Auto-Cal Done",			_wait_gsc_auto_cal_done,		{ 0, 0,			ADADIO_WAIT_GSC_AUTO_CAL_DONE,		0, 0,		_20_SEC	}	},
	{ "Input Buffer Empty",		_wait_gsc_ain_buf_empty,		{ 0, 0,			ADADIO_WAIT_GSC_AIN_BUF_EMPTY,		0, 0,		_20_SEC	}	},
	{ "Input Buffer Half Full",	_wait_gsc_ain_buf_half_full,	{ 0, 0,			ADADIO_WAIT_GSC_AIN_BUF_HALF_FULL,	0, 0,		_20_SEC	}	},
	{ "Input Buffer Full",		_wait_gsc_ain_buf_full,			{ 0, 0,			ADADIO_WAIT_GSC_AIN_BUF_FULL,		0, 0,		_20_SEC	}	},
	{ "AI Burst Done",			_wait_gsc_ain_burst_done,		{ 0, 0,			ADADIO_WAIT_GSC_AIN_BURST_DONE,		0, 0,		_20_SEC	}	},
	{ "AO Strobe Done",			_wait_gsc_aout_strobe_done,		{ 0, 0,			ADADIO_WAIT_GSC_AOUT_STROBE_DONE,	0, 0,		_20_SEC	}	},

	// name						function						flags ...		io												timeout_ms
	{ "Rx Done",				_wait_io_rx_done,				{ 0, 0, 0, 0,	ADADIO_WAIT_IO_RX_DONE,							_20_SEC	}	},
	{ "Rx Error",				_wait_io_rx_error,				{ 0, 0, 0, 0,	ADADIO_WAIT_IO_RX_ERROR,						_20_SEC	}	},
	{ "Rx Timeout",				_wait_io_rx_timeout,			{ 0, 0, 0, 0,	ADADIO_WAIT_IO_RX_TIMEOUT,						_20_SEC	}	},
	{ "Rx Abort",				_wait_io_rx_abort,				{ 0, 0, 0, 0,	ADADIO_WAIT_IO_RX_ABORT,						_20_SEC	}	},

	// function					flags ...																						timeout_ms
	{ "Wait Timeout",			_wait_timeout,					{ 0, 0, 0, 0,	ADADIO_WAIT_IO_RX_DONE,							_1_SEC	}	},
	{ "Wait Cancel",			_wait_cancel,					{ 0, 0, 0, 0,	ADADIO_WAIT_IO_RX_DONE,							_30_SEC	}	},

	// terminate list
	{ NULL, NULL, { 1 } }
};



//*****************************************************************************
static int _wait_thread(void* arg)
{
	int	ret;

	sem_post(&sem_enter);
	ret	= adadio_ioctl(wait_fd, ADADIO_IOCTL_WAIT_EVENT, &wait_dst);

	if (ret < 0)
		wait_dst.flags	= 0xFFFFFFFF;

	sem_post(&sem_exit);
	return(0);
}


//*****************************************************************************
static int _process_thread(int index)
{
	char		buf[64];
	int			errs	= 0;
	int			i;
	int			ret;
	int			status;
	os_thread_t	thread;
	gsc_wait_t	wait;

	sem_init(&sem_enter, 0, 0);
	sem_init(&sem_exit, 0, 0);
	memset(&thread, 0, sizeof(thread));
	sprintf(buf, "wait event %d", index);
	wait_dst	= wait_src[index].wait;
	errs		+= os_thread_create(&thread, buf, _wait_thread, NULL);

	// Wait for the thread to become waiting.
	sem_wait(&sem_enter);

	if (errs == 0)
	{
		wait	= wait_src[index].wait;

		for (i = 0; i < 100; i++)
		{
			ret	= adadio_ioctl(wait_fd, ADADIO_IOCTL_WAIT_STATUS, &wait);

			if (ret < 0)
			{
				printf(	"FAIL <---  (%d. status request %d: error %d)\n",
						__LINE__,
						index,
						ret);
				errs++;
				break;
			}

			if (wait.count == 1)
				break;

			os_sleep_ms(100);
		}
	}

	// Initiate the respective event.
	status	= wait_src[index].function(wait_fd);

	if (status < 0)
	{
	}
	else
	{
		errs	+= status;

		// Verify that the wait ended as expected.

		if (wait_src[index].wait.flags == 0xFFFFFFFF)
		{
		}
		else
		{
			// Wait for the resumed thread to run.
			sem_wait(&sem_exit);

			for (i = 0;; i++)
			{
				if (wait_dst.flags)
					break;

				if (i >= 100)
				{
					printf(	"FAIL <---  (%d. thread failed to resume)\n",
							__LINE__);
					errs++;
					break;
				}

				os_sleep_ms(100);
			}

			if (wait_src[index].wait.timeout_ms == _1_SEC)
			{
				if (wait_dst.flags != GSC_WAIT_FLAG_TIMEOUT)
				{
					printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
							__LINE__,
							(long) GSC_WAIT_FLAG_TIMEOUT,
							(long) wait_dst.flags);
					errs++;
				}
			}
			else if (wait_src[index].wait.timeout_ms == _20_SEC)
			{
				if (wait_dst.flags != GSC_WAIT_FLAG_DONE)
				{
					printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
							__LINE__,
							(long) GSC_WAIT_FLAG_DONE,
							(long) wait_dst.flags);
					errs++;
				}
			}
			else if (wait_src[index].wait.timeout_ms == _30_SEC)
			{
				if (wait_dst.flags != GSC_WAIT_FLAG_CANCEL)
				{
					printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
							__LINE__,
							(long) GSC_WAIT_FLAG_TIMEOUT,
							(long) wait_dst.flags);
					errs++;
				}
			}
			else
			{
				printf("FAIL <---  (%d. INTERNAL ERROR)\n", __LINE__);
				errs++;
			}
		}
	}

	wait.flags			= 0;
	wait.main			= 0xFFFFFFFF;
	wait.gsc			= 0xFFFFFFFF;
	wait.alt			= 0xFFFFFFFF;
	wait.io				= 0xFFFFFFFF;
	wait.timeout_ms		= 0;
	wait.count			= 0;
	ret		= adadio_ioctl(wait_fd, ADADIO_IOCTL_WAIT_CANCEL, &wait);
	errs	+= ret ? 1 : 0;
	errs	+= os_thread_destroy(&thread);

	if ((status >= 0) && (errs == 0))
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;
	int	i;

	printf("\n");
	gsc_label_level_inc();

	// Create threads awaiting each of the source crtiteria
	// and verify that it resumes when the criteria is met.

	for (i = 0; wait_src[i].wait.flags == 0; i++)
	{
		gsc_label(wait_src[i].name);
		errs	+= adadio_initialize(fd, -1, 0);

		wait_fd	= fd;
		memset(&wait_dst, 0, sizeof(wait_dst));
		errs	+= _process_thread(i);

		errs	+= adadio_initialize(fd, -1, 0);
	}

	gsc_label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	wait_event_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service ADADIO_IOCTL_WAIT_EVENT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int wait_event_test(int fd)
{
	int	errs	= 0;

	gsc_label("ADADIO_IOCTL_WAIT_EVENT");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	return(errs);
}


