// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/utils/util_config_ai.c $
// $Rev: 52440 $
// $Date: 2023-01-30 13:19:25 -0600 (Mon, 30 Jan 2023) $

// ADADIO: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	adadio_config_ai
*
*	Purpose:
*
*		Configure the basic services.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	adadio_config_ai(int fd, int index, int verbose)
{
	int	errs	= 0;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		gsc_label_level_inc();
		printf("\n");
	}

	errs	+= adadio_initialize		(fd, index, verbose);
	errs	+= adadio_rx_io_mode		(fd, index, verbose, GSC_IO_MODE_PIO,				NULL);
	errs	+= adadio_rx_io_timeout		(fd, index, verbose, ADADIO_IO_TIMEOUT_DEFAULT,		NULL);

	errs	+= adadio_auto_calibrate	(fd, index, verbose);	// Cable may need disconnecting for this.

	errs	+= adadio_ain_nrate			(fd, index, verbose, 400,							NULL);
	errs	+= adadio_ain_mode			(fd, index, verbose, ADADIO_AIN_MODE_DIFF_CONT,		NULL);
	errs	+= adadio_data_format		(fd, index, verbose, ADADIO_DATA_FORMAT_OFF_BIN,	NULL);
	errs	+= adadio_ain_chan_last		(fd, index, verbose, 7,								NULL);
	errs	+= adadio_ain_buf_size		(fd, index, verbose, ADADIO_AIN_BUF_SIZE_32768,		NULL);

	errs	+= adadio_aout_enable		(fd, index, verbose, ADADIO_AOUT_ENABLE_NO,			NULL);
	errs	+= adadio_aout_ch_0_write	(fd, index, verbose, 0xFC,							NULL);
	errs	+= adadio_aout_ch_1_write	(fd, index, verbose, 0xFC,							NULL);
	errs	+= adadio_aout_ch_2_write	(fd, index, verbose, 0xFC,							NULL);
	errs	+= adadio_aout_ch_3_write	(fd, index, verbose, 0xFC,							NULL);
	errs	+= adadio_aout_strobe_enable(fd, index, verbose, ADADIO_AOUT_STROBE_ENABLE_NO,	NULL);

	errs	+= adadio_irq_sel			(fd, index, verbose, ADADIO_IRQ_INIT_DONE,			NULL);
	errs	+= adadio_loopback_channel	(fd, index, verbose, 0,								NULL);

	errs	+= adadio_dio_pin_write		(fd, index, verbose, 0,								NULL);
	errs	+= adadio_dio_port_dir		(fd, index, verbose, ADADIO_DIO_PORT_DIR_INPUT,		NULL);
	errs	+= adadio_dio_port_write	(fd, index, verbose, 0,								NULL);

	errs	+= adadio_ain_buf_clear		(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


