// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/utils/util_irq_sel.c $
// $Rev: 44404 $
// $Date: 2019-02-06 13:40:20 -0600 (Wed, 06 Feb 2019) $

// ADADIO: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	adadio_irq_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the ADADIO_IOCTL_IRQ_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	adadio_irq_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	int		ret;

	if (verbose)
		gsc_label_index("IRQ Selection", index);

	ret		= adadio_ioctl(fd, ADADIO_IOCTL_IRQ_SEL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case ADADIO_IRQ_INIT_DONE:

			strcpy(buf, "Initialize Done");
			break;

		case ADADIO_IRQ_AUTO_CAL_DONE:

			strcpy(buf, "Auto-Calibration Done");
			break;

		case ADADIO_IRQ_AIN_BUF_EMPTY:

			strcpy(buf, "Input Buffer Empty");
			break;

		case ADADIO_IRQ_AIN_BUF_HALF_FULL:

			strcpy(buf, "Buffer Half Full");
			break;

		case ADADIO_IRQ_AIN_BUF_FULL:

			strcpy(buf, "Buffer Full");
			break;

		case ADADIO_IRQ_AIN_BURST_DONE:

			strcpy(buf, "Input Burst Done");
			break;

		case ADADIO_IRQ_AOUT_STROBE_DONE:

			strcpy(buf, "Output Strobe done");
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);

	if (get)
		get[0]	= set;

	return(errs);
}


