// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/utils/util_reg.c $
// $Rev: 52440 $
// $Date: 2023-01-30 13:19:25 -0600 (Mon, 30 Jan 2023) $

// ADADIO: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)					"GSC " #a, ADADIO_GSC_##a, 0, 0
#define	SHOW_FIELD(f,l,e,ls,d)		gsc_reg_field_show(width+10,24, value, (f),(l),(e),(ls),(d))
#define	SIZEOF_ARRAY(a)				(sizeof((a)) / sizeof((a)[0]))



//*****************************************************************************
static int _brr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	model[]	=
	{
		"ADADIO",
		"ADADIO2"
	};

	static const char*	rate[]	=
	{
		"100K S/S",
		"200K S/S"
	};

	static const char*	volt[]	=
	{
		"+-10V",
		"+-5V",
		"+-2.5V",
		"RESERVED"
	};

	SHOW_FIELD(15,  0, 1, NULL,		"Firmware Revision"	);
	SHOW_FIELD(13, 13, 1, model,	"Device Model"		);
	SHOW_FIELD(16, 16, 1, rate,		"Max Sample Rate"	);
	SHOW_FIELD(18, 17, 1, volt,		"Voltage Range"		);
	SHOW_FIELD(31, 19, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _bcr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	aim[]	=
	{
		"Single Ended, Continuous",
		"Single Ended, Burst",
		"Differential, Continuous",
		"Differential, Burst",
		"Loop Back Test",
		"+Vref Test",
		"RESERVED",
		"Zero Test"
	};

	static const char*	cal_mode[]	=
	{
		"Normal",
		"Load from EEPROM",
		"Auto-Calibration",
		"RESERVED"
	};

	static const char*	irq[]	=
	{
		"Initialization Done",
		"Auto-Calibration Done",
		"Input Buffer Empty",
		"Input Buffer Half Full",
		"Input Buffer Full",
		"Input Burst Done",
		"Output Strobe Done",
		"RESERVED"
	};

	static const char*	last[]	=
	{
		"0",
		"1",
		"2",
		"3",
		"4",
		"5",
		"6",
		"7"
	};

	static const char*	lbc[]	=
	{
		"Output Channel 0",
		"Output Channel 1",
		"Output Channel 2",
		"Output Channel 3"
	};

	static const char*	size[]	=
	{
		"1",
		"2",
		"4",
		"8",
		"16",
		"32",
		"64",
		"128",
		"256",
		"512",
		"1024",
		"2048",
		"4096",
		"8192",
		"16384",
		"32768"
	};

	static const char*	disable[]	= { "Disable",			"Enable"		};
	static const char*	idle[]		= { "Idle",				"Active"		};
	static const char*	negated[]	= { "Negated",			"Asserted"		};
	static const char*	no[]		= { "No",				"Yes"			};
	static const char*	no_dis[]	= { "No (Disabled)",	"Yes (Enabled)"	};
	static const char*	pass[]		= { "Pass",				"Fail"			};
	static const char*	twos[]		= { "Twos Compliment",	"Offset Binary"	};

	SHOW_FIELD( 2,  0, 1, aim,		"Analog Input Mode"			);
	SHOW_FIELD( 4,  3, 1, lbc,		"Loop Back Channel"			);
	SHOW_FIELD( 5,  5, 1, pass,		"Auto-Cal Status"			);
	SHOW_FIELD( 6,  6, 1, twos,		"Data Format"				);
	SHOW_FIELD(10,  7, 1, size,		"Buffer Size"				);
	SHOW_FIELD(11, 11, 1, idle,		"Clear Buffer"				);
	SHOW_FIELD(14, 12, 1, NULL,		"Reserved"					);
	SHOW_FIELD(17, 15, 1, last,		"Last Channel"				);
	SHOW_FIELD(18, 18, 1, disable,	"Output Enable"				);
	SHOW_FIELD(19, 19, 1, no_dis,	"Output Strobe Enable"		);
	SHOW_FIELD(20, 20, 1, no,		"Input Buffer Empty"		);
	SHOW_FIELD(21, 21, 1, no,		"Input Buffer Half Full"	);
	SHOW_FIELD(22, 22, 1, no,		"Input Buffer Full"			);
	SHOW_FIELD(25, 23, 1, irq,		"IRQ Select"				);
	SHOW_FIELD(26, 26, 1, negated,	"IRQ Status"				);
	SHOW_FIELD(28, 27, 1, cal_mode,	"Calibration Mode"			);
	SHOW_FIELD(29, 29, 1, idle,		"Output Strobe"				);
	SHOW_FIELD(30, 30, 1, idle,		"Input Trigger"				);
	SHOW_FIELD(31, 31, 1, idle,		"Initialize"				);
	return(0);
}



//*****************************************************************************
static int _diopr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	input[]	= { "Input",		"Output"	};
	static const char*	zero[]	= { "0 (Clear)",	"1 (Set)"	};

	SHOW_FIELD( 7,  0, 1, NULL,		"I/O Data"			);
	SHOW_FIELD( 8,  8, 1, zero,		"Dedicated Input"	);
	SHOW_FIELD( 9,  9, 1, zero,		"Dedicated Output"	);
	SHOW_FIELD(10, 10, 1, input,	"Data Direction"	);
	SHOW_FIELD(31, 11, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _aocr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(15,  0, 1, NULL,	"Data"		);
	SHOW_FIELD(31, 16, 1, NULL,	"Inactive"	);
	return(0);
}



//*****************************************************************************
static int _aidr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(15,  0, 1, NULL,	"Data"		);
	SHOW_FIELD(31, 16, 1, NULL,	"Extension"	);
	return(0);
}



//*****************************************************************************
static int _srr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(15,  0, 1, NULL,	"Nrate"		);
	SHOW_FIELD(31, 16, 1, NULL,	"Inactive"	);
	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(BRR),	0,	_brr_decode,	"Board Revision Register"		},
	{ _GSC_REG(BCR),	0,	_bcr_decode,	"Board Control Register"		},
	{ _GSC_REG(DIOPR),	0,	_diopr_decode,	"Digital I/O Port Register"		},
	{ _GSC_REG(AOC0R),	0,	_aocr_decode,	"Analog Output Ch 0 Register"	},
	{ _GSC_REG(AOC1R),	0,	_aocr_decode,	"Analog Output Ch 1 Register"	},
	{ _GSC_REG(AOC2R),	0,	_aocr_decode,	"Analog Output Ch 2 Register"	},
	{ _GSC_REG(AOC3R),	0,	_aocr_decode,	"Analog Output Ch 3 Register"	},
	{ _GSC_REG(AIDR),	0,	_aidr_decode,	"Analog Input Data Register"	},
	{ _GSC_REG(SRR),	0,	_srr_decode,	"Sample Rate Register"			},

	{ NULL, 0, 0, 0, 0, NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	adadio_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* adadio_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	adadio_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* adadio_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	adadio_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* adadio_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	adadio_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* adadio_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	adadio_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int adadio_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, adadio_reg_read);
	return(errs);
}


