// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/ADADIO/ADADIO_Linux_3.x.x.x_GSC_DN/utils/util_rx_io_timeout.c $
// $Rev: 52440 $
// $Date: 2023-01-30 13:19:25 -0600 (Mon, 30 Jan 2023) $

// ADADIO: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	adadio_rx_io_timeout
*
*	Purpose:
*
*		Provide a visual wrapper for the ADADIO_IOCTL_RX_IO_TIMEOUT service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	adadio_rx_io_timeout(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Rx I/O Timeout", index);

	ret		= adadio_ioctl(fd, ADADIO_IOCTL_RX_IO_TIMEOUT, &set);
	errs	= ret ? 1 : 0;

	if (verbose)
	{
		if (set == -1)
		{
			ptr	= "This service is not supported by this device.";
		}
		else if (set == ADADIO_IO_TIMEOUT_INFINITE)
		{
			ptr	= "infinite/never timeout";
		}
		else
		{
			sprintf(buf, "%ld second%s", (long) set, (set == 1) ? "" : "s");
			ptr	= buf;
		}

		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


